part of katana_model_openapi_builder;

/// Generator that automatically creates converted code from OpenAPI.
///
/// OpenAPIから変換されたコードを自動作成するジェネレーター。
class OpenAPIGenerator {
  /// Generator that automatically creates converted code from OpenAPI.
  ///
  /// OpenAPIから変換されたコードを自動作成するジェネレーター。
  OpenAPIGenerator(
    this.api,
    this.baseName,
    this.partFileName,
    this.freezedPartFileName,
  );

  /// Actual data from OepnAPI.
  ///
  /// OepnAPIの実データ。
  final APIDocument api;

  /// Base name of the file.
  ///
  /// ファイルのベースネーム。
  final String baseName;

  /// Name of the part file.
  ///
  /// パートファイルの名前。
  final String partFileName;

  /// Name of the file to be generated by freezed.
  ///
  /// freezedで生成されるファイルの名前。
  final String freezedPartFileName;

  /// Generates a library.
  ///
  /// ライブラリーを生成します。
  FutureOr<Library> generate() async {
    final schemas = (api.components?.schemas ?? {})
        .map((key, value) => MapEntry(key.toPascalCase(), value))
      ..removeWhere((key, value) => value == null);
    final builder = LibraryBuilder()
      ..directives.addAll([
        Directive.import("package:freezed_annotation/freezed_annotation.dart"),
        Directive.part(partFileName),
        Directive.part(freezedPartFileName),
      ])
      ..body.addAll([
        ...freezedClass(schemas.cast()),
      ]);

    return builder.build();
  }
}
